import os
import shutil

from pacu.core.base import Base, engine
from pacu.utils import set_sigint_handler


def setup_database_if_not_present(database_file_path: str, auto_proceed: bool=True) -> bool:
    if os.path.exists(database_file_path):
        # rename the database if it's an old version to avoid sqlalchemy
        # error like "no such column: ..." and to create a new database
        # with new schema
        if os.path.exists(os.path.join(os.path.dirname(database_file_path), 'version')):
            db_version=open(os.path.join(os.path.dirname(database_file_path), 'version'), 'r').read()
        else:
            db_version='old'
        pacu_version=open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'version')).read()
        if db_version != pacu_version:
            shutil.move(database_file_path,
                    os.path.join(os.path.dirname(database_file_path),
                    'sqlite-backup.db'))
            print('The database schema has changed. A new database will be created')
            print('The old database has be renamed sqlite-backup.db')
            return attempt_to_create_database(database_file_path, auto_proceed)
        else:
            return True
    else:
        print('No database found at {}'.format(database_file_path))
        return attempt_to_create_database(database_file_path, auto_proceed)


def attempt_to_create_database(database_file_path, auto_proceed=True):
    if auto_proceed:
        proceed = 'y'
    else:
        print('A database will be created at the following location:\n    {}'.format(database_file_path))

        if os.path.exists(database_file_path):
            question = 'A file already exists at this location.\nAre you sure you want to delete and recreate it? [y/n]\n> '
        else:
            question = 'Create a new database file? [y/n]\n> '

        proceed = input(question)

    if proceed.strip().lower() == 'y':
        if os.path.exists(database_file_path):
            os.remove(database_file_path)

        # Base.metadata.create_all requires all models to be loaded before
        # tables can be created. It's is placed here for emphasis.
        from pacu.core.models import AWSKey, PacuSession
        Base.metadata.create_all(engine)

        # copy the file with the DB version in local directory
        shutil.copyfile(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'version'), os.path.join(os.path.dirname(database_file_path), 'version'))

        print('Database created at {}\n'.format(database_file_path))
        return True

    else:
        print('Database creation cancelled.\n')
        return False


if __name__ == '__main__':
    from pacu.settings import DATABASE_FILE_PATH
    set_sigint_handler(exit_text='\nDatabase creation cancelled.')
    attempt_to_create_database(DATABASE_FILE_PATH, auto_proceed=False)
